@@ -128,6 +128,22 @@
 # ### 3.0.0 -- 2022-04-19
 #
 # * Major rewrite, use rlang compatible error objects. New API.
+#
+# ### 3.0.1 -- 2022-06-17
+#
+# * Remove the `rlang_error` and `rlang_trace` classes, because our new
+#   deparsed `call` column in the trace is not compatible with rlang.
+#
+# ### 3.0.2 -- 2022-08-01
+#
+# * Use a `procsrcref` column for processed source references.
+#   Otherwise testthat (and probably other rlang based packages), will
+#   pick up the `srcref` column, and they expect an `srcref` object there.
+#
+# ### 3.1.0 -- 2022-10-04
+#
+# * Add ANSI hyperlinks to stack traces, if we have a recent enough
+#   cli package that supports this.
 
 err <- local({
 
@@ -174,7 +190,7 @@ err <- local({
 
   new_error <- function(..., call. = TRUE, srcref = NULL, domain = NA) {
     cond <- new_cond(..., call. = call., domain = domain, srcref = srcref)
-    class(cond) <- c("rlib_error_3_0", "rlib_error", "rlang_error", "error", "condition")
+    class(cond) <- c("rlib_error_3_0", "rlib_error", "error", "condition")
     cond
   }
 
@@ -298,15 +314,16 @@ err <- local({
   #' @param err Error object or message to use for the child error.
   #' @param call Call to use in the re-thrown error. See [throw()].
 
-  chain_error <- function(expr, err, call = sys.call(-1)) {
+  chain_error <- function(expr, err, call = sys.call(-1), srcref = NULL) {
     .hide_from_trace <- 1
     force(call)
-    srcref <- utils::getSrcref(sys.call())
+    srcref <- srcref %||% utils::getSrcref(sys.call())
     withCallingHandlers({
       expr
     }, error = function(e) {
       .hide_from_trace <- 0:1
       e$srcref <- srcref
+      e$procsrcref <- NULL
       if (!inherits(err, "condition")) {
         err <- new_error(err, call. = call)
       }
@@ -337,11 +354,12 @@ err <- local({
       error = function(e) {
         .hide_from_trace <- 0:1
         e$srcref <- srcref
+        e$procsrcref <- NULL
         e$call <- call
         name <- native_name(.NAME)
         err <- new_error("Native call to `", name, "` failed", call. = call1)
         cerror <- if (inherits(e, "simpleError")) "c_error"
-        class(err) <- c(cerror, "rlib_error_3_0", "rlib_error", "rlang_error", "error", "condition")
+        class(err) <- c(cerror, "rlib_error_3_0", "rlib_error", "error", "condition")
         throw_error(err, parent = e)
       }
     )
@@ -371,11 +389,12 @@ err <- local({
       error = function(e) {
         .hide_from_trace <- 0:1
         e$srcref <- srcref
+        e$procsrcref <- NULL
         e$call <- call
         name <- native_name(.NAME)
         err <- new_error("Native call to `", name, "` failed", call. = call1)
         cerror <- if (inherits(e, "simpleError")) "c_error"
-        class(err) <- c(cerror, "rlib_error_3_0", "rlib_error", "rlang_error", "error", "condition")
+        class(err) <- c(cerror, "rlib_error_3_0", "rlib_error", "error", "condition")
         throw_error(err, parent = e)
       }
     )
@@ -404,7 +423,11 @@ err <- local({
     namespaces <- unlist(lapply(
       seq_along(frames),
       function(i) {
-        env_label(topenvx(environment(sys.function(i))))
+        if (is_operator(calls[[i]])) {
+          "o"
+        } else {
+          env_label(topenvx(environment(sys.function(i))))
+        }
       }
     ))
     pids <- rep(cond$`_pid` %||% Sys.getpid(), length(calls))
@@ -438,6 +461,7 @@ err <- local({
     pcs <- lapply(calls, function(c) process_call(list(call = c)))
     calls <- lapply(pcs, "[[", "call")
     srcrefs <- I(lapply(pcs, "[[", "srcref"))
+    procsrcrefs <- I(lapply(pcs, "[[", "procsrcref"))
 
     cond$trace <- new_trace(
       calls,
@@ -446,12 +470,18 @@ err <- local({
       namespaces = namespaces,
       scopes = scopes,
       srcrefs = srcrefs,
+      procsrcrefs = procsrcrefs,
       pids
     )
 
     cond
   }
 
+  is_operator <- function(cl) {
+    is.call(cl) && length(cl) >= 1 && is.symbol(cl[[1]]) &&
+      grepl("^[^.a-zA-Z]", as.character(cl[[1]]))
+  }
+
   mark_invisible_frames <- function(funs, frames) {
     visibles <- rep(TRUE, length(frames))
     hide <- lapply(frames, "[[", ".hide_from_trace")
@@ -522,7 +552,7 @@ err <- local({
     topenv(x, matchThisEnv = err_env)
   }
 
-  new_trace <- function (calls, parents, visibles, namespaces, scopes, srcrefs, pids) {
+  new_trace <- function (calls, parents, visibles, namespaces, scopes, srcrefs, procsrcrefs, pids) {
     trace <- data.frame(
       stringsAsFactors = FALSE,
       parent = parents,
@@ -530,11 +560,12 @@ err <- local({
       namespace = namespaces,
       scope = scopes,
       srcref = srcrefs,
+      procsrcref = procsrcrefs,
       pid = pids
     )
     trace$call <- calls
 
-    class(trace) <- c("rlib_trace_3_0", "rlang_trace", "rlib_trace", "tbl", "data.frame")
+    class(trace) <- c("rlib_trace_3_0", "rlib_trace", "tbl", "data.frame")
     trace
   }
 
@@ -752,7 +783,7 @@ err <- local({
   format_header_line_cli <- function(x, prefix = NULL) {
     p_error <- format_error_heading_cli(x, prefix)
     p_call <- format_call_cli(x$call)
-    p_srcref <- format_srcref_cli(conditionCall(x), x$srcref)
+    p_srcref <- format_srcref_cli(conditionCall(x), x$procsrcref %||% x$srcref)
     paste0(p_error, p_call, p_srcref, if (!is.null(conditionCall(x))) ":")
   }
 
@@ -790,12 +821,18 @@ err <- local({
     if (is.null(ref)) return("")
 
     link <- if (ref$file != "") {
-      cli::style_hyperlink(
-        cli::format_inline("{basename(ref$file)}:{ref$line}:{ref$col}"),
-        paste0("file://", ref$file),
-        params = c(line = ref$line, col = ref$col)
-      )
-
+      if (Sys.getenv("R_CLI_HYPERLINK_STYLE") == "iterm") {
+        cli::style_hyperlink(
+          cli::format_inline("{basename(ref$file)}:{ref$line}:{ref$col}"),
+          paste0("file://", ref$file, "#", ref$line, ":", ref$col)
+        )
+      } else {
+        cli::style_hyperlink(
+          cli::format_inline("{basename(ref$file)}:{ref$line}:{ref$col}"),
+          paste0("file://", ref$file),
+          params = c(line = ref$line, col = ref$col)
+        )
+      }
     } else {
       paste0("line ", ref$line)
     }
@@ -824,10 +861,10 @@ err <- local({
       rep(TRUE, nrow(x))
     }
 
-    srcref <- if ("srcref" %in% names(x)) {
+    srcref <- if ("srcref" %in% names(x) || "procsrcref" %in% names(x)) {
       vapply(
         seq_len(nrow(x)),
-        function(i) format_srcref_cli(x$call[[i]], x$srcref[[i]]),
+        function(i) format_srcref_cli(x$call[[i]], x$procsrcref[[i]] %||% x$srcref[[i]]),
         character(1)
       )
     } else {
@@ -838,7 +875,9 @@ err <- local({
       cli::col_silver(format(x$num), ". "),
       ifelse (visible, "", "| "),
       scope,
-      vapply(x$call, format_trace_call_cli, character(1)),
+      vapply(seq_along(x$call), function(i) {
+        format_trace_call_cli(x$call[[i]], x$namespace[[i]])
+      }, character(1)),
       srcref
     )
 
@@ -850,10 +889,16 @@ err <- local({
     lines
   }
 
-  format_trace_call_cli <- function(call) {
+  format_trace_call_cli <- function(call, ns = "") {
+    envir <- tryCatch(asNamespace(ns), error = function(e) .GlobalEnv)
     cl <- trimws(format(call))
     if (length(cl) > 1) { cl <- paste0(cl[1], " ", cli::symbol$ellipsis) }
-    fmc <- cli::code_highlight(cl)[1]
+    # Older cli does not have 'envir'.
+    if ("envir" %in% names(formals(cli::code_highlight))) {
+      fmc <- cli::code_highlight(cl, envir = envir)[1]
+    } else {
+      fmc <- cli::code_highlight(cl)[1]
+    }
     cli::ansi_strtrim(fmc, cli::console_width() - 5)
   }
 
@@ -892,10 +937,10 @@ err <- local({
       rep(TRUE, nrow(x))
     }
 
-    srcref <- if ("srcref" %in% names(x)) {
+    srcref <- if ("srcref" %in% names(x) || "procsrfref" %in% names(x)) {
       vapply(
         seq_len(nrow(x)),
-        function(i) format_srcref_plain(x$call[[i]], x$srcref[[i]]),
+        function(i) format_srcref_plain(x$call[[i]], x$procsrcref[[i]] %||% x$srcref[[i]]),
         character(1)
       )
     } else {
@@ -920,7 +965,7 @@ err <- local({
   format_header_line_plain <- function(x, prefix = NULL) {
     p_error <- format_error_heading_plain(x, prefix)
     p_call <- format_call_plain(x$call)
-    p_srcref <- format_srcref_plain(conditionCall(x), x$srcref)
+    p_srcref <- format_srcref_plain(conditionCall(x), x$procsrcref %||% x$srcref)
     paste0(p_error, p_call, p_srcref, if (!is.null(conditionCall(x))) ":")
   }
 
@@ -985,7 +1030,7 @@ err <- local({
   }
 
   process_call <- function(cond) {
-    cond[c("call", "srcref")] <- list(
+    cond[c("call", "srcref", "procsrcref")] <- list(
       call = if (is.null(cond$call)) {
         NULL
       } else if (is.character(cond$call)) {
@@ -993,7 +1038,8 @@ err <- local({
       } else {
         deparse(cond$call, nlines = 2)
       },
-      srcref = get_srcref(cond$call, cond$srcref)
+      srcref = NULL,
+      procsrcref = get_srcref(cond$call, cond$procsrcref %||% cond$srcref)
     )
     cond
   }
